/* lmdif0.c -- driver for lmdif */
#include <malloc.h>
#include "xfloat.h"
#include "nlls.h"

//
// This port of lmdif from minpack provides a 'masking' function
// which allows the user to set up an approximating function with
// adjustable parameters, and suppress the adjustment of selected
// members of the parameter set. Thus it is possible to hold
// certain parameters constant while leaving the others free to
// vary. This feature is desirable if specific parameters have
// known (or required) value, or if several parameters are
// correlated and it is necessary to vary them one at a time.
//
int lmdif0(void fcn(int,int,xfloat *,xfloat *,int *),int m, int n,
    xfloat *x,int msk[],xfloat *fvec,xfloat tol,int *info)
{
    int j,maxfev,mode,nfev;
    int *ipvt;
    xfloat ftol,xtol,gtol,epsfcn,factor,fnorm;
    xfloat *diag,**fjac,*qtf,*wa1,*wa2,*wa3,*wa4;

/* Check input parameters */
   if (n <= 0 || m < n || tol < 0.0) {
        *info = 0;
        return(1);
   }
/* Allocate memory for working arrays. */
    ipvt = (int *)calloc(n,sizeof(int));
    diag = (xfloat *)calloc(n,sizeof(xfloat));
    qtf = (xfloat *)calloc(n,sizeof(xfloat));
    wa1 = (xfloat *)calloc(n,sizeof(xfloat));
    wa2 = (xfloat *)calloc(n,sizeof(xfloat));
    wa3 = (xfloat *)calloc(n,sizeof(xfloat));
    wa4 = (xfloat *)calloc(m,sizeof(xfloat));


/* Create 2d matrix for Jacobian */
    fjac = (xfloat **)calloc(n,sizeof(xfloat *));
   for (j=0;j<n;j++)
        fjac[j] = (xfloat *)calloc(m,sizeof(xfloat));

/* Set convergence tolerances */
    ftol = tol;
    xtol = tol;
    gtol = 0.0;

    maxfev = 800;
    epsfcn = 0.0;
    mode = 1;
    factor = 10.0;

    lmdif(fcn,m,n,x,msk,fvec,ftol,xtol,gtol,maxfev,epsfcn,diag,mode,
        factor,info,&nfev,fjac,ipvt,qtf,wa1,wa2,wa3,wa4);

    if (*info == 8) *info = 4;
    for (j=0;j<n;j++)
        free(fjac[j]);
    free(fjac);
    free(wa4);
    free(wa3);
    free(wa2);
    free(wa1);
    free(qtf);
    free(diag);
    free(ipvt);
    return(0);
}
    



    
