/* xlmd.cpp -- test driver for lmdif */
#include "xfloat.h"
#include "dpmpar.h"
#include "nlls.h"

/* N = number of parameters in model, M = number of measured points to match. */
#define N       3
#define M       7

/* Array of 'measured' points. */
static xfloat lorz[M] = {0.2,
                       0.4,
                       1.0,
                       2.0,
                       1.0,
                       0.4,
                       0.2};
/* Parameter list with initial estimates. */
static xfloat x[N] = { 1.0,
                        1.0,
                        2.0};
static int msk[N] = { 1, 1, 1};
/* Function to calculate residuals: r = (model estimate - measured value).
 * Note that residuals are in fvec[i].
 * This test model is a simple Lorentzian with x[0] controlling the
 * amplitude, x[1] controlling horizontal scale, and
 * x[2] controlling the horizontal offset.
 */
xfloat fvec[M] = {0.0};
void fcn(int m,int n,xfloat x[],xfloat *fvec,int *iflag)
{
    int i;
    xfloat tmp;
    
    for (i = 0; i < m; i++) {
        tmp = x[2] - (xfloat)i;
        fvec[i] = (x[0] / (1.0 + x[1] * tmp * tmp))-lorz[i];
    }
}

void main()
{
    int m,n,info,i,ecode;
    xfloat tol,fnorm,tmp;

    m = M;
    n = N;

/* Set convergence tolerances */
//    tol = sqrt(dpmpar[0]);
    tol = dpmpar[0];
    cerr << "Tolerance: " << tol << endl;
    cerr << "Model: A/(1+B*(x-C)^2)" << endl;
    cerr << "Target: 2/(1+(x-3)^2) -> A=2, B=1, C=3\n" << endl;
    cerr << "Starting estimate are: A=1, B=1, C=2 " << endl << endl;
    ecode=lmdif0(fcn,m,n,x,msk,fvec,tol,&info);
    fnorm = enorm(m,fvec);
    cout << "Done!" << endl;
    cout << "L2 norm, " << fnorm << endl;
    cout << "Exit parameter, " << info << endl;
    cout << "Final approximate solution, " << endl;
    cout << "A = " << x[0] << endl;
    cout << "B = " << x[1] << endl;
    cout << "C = " << x[2] << endl;
    cout << "Residuals:" << endl;
    for (i=0;i<m;i++) {
        cout << "fvec[" << i << "] = " << fvec[i] << endl;
    }
}

